# external
import pytest

# project
import textdistance

ALG = textdistance.DamerauLevenshtein

COMMON = [
    ('test', 'text', 1),
    ('test', 'tset', 1),
    ('test', 'qwy', 4),
    ('test', 'testit', 2),
    ('test', 'tesst', 1),
    ('test', 'tet', 1),

    ('cat', 'hat', 1),
    ('Niall', 'Neil', 3),
    ('aluminum', 'Catalan', 7),
    ('ATCG', 'TAGC', 2),

    ('ab', 'ba', 1),
    ('ab', 'cde', 3),
    ('ab', 'ac', 1),
    ('ab', 'bc', 2),
]


@pytest.mark.parametrize('left, right, expected', COMMON + [
    ('ab', 'bca', 3),
    ('abcd', 'bdac', 4),
])
def test_distance_restricted(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected

    actual = ALG()._pure_python_restricted(left, right)
    assert actual == expected


@pytest.mark.parametrize('left, right, expected', COMMON + [
    ('ab', 'bca', 2),
    ('abcd', 'bdac', 3),
])
def test_distance_unrestricted(left, right, expected):
    actual = ALG(external=False, restricted=False)(left, right)
    assert actual == expected

    actual = ALG(external=True, restricted=False)(left, right)
    assert actual == expected

    actual = ALG()._pure_python_unrestricted(left, right)
    assert actual == expected



# external
import pytest

# project
import textdistance


ALG = textdistance.LCSSeq


@pytest.mark.parametrize('left, right, expected', [
    ('ab', 'cd', ''),
    ('abcd', 'abcd', 'abcd'),

    ('test', 'text', 'tet'),
    ('thisisatest', 'testing123testing', 'tsitest'),
    ('DIXON', 'DICKSONX', 'DION'),
    ('random exponential', 'layer activation', 'ratia'),

    ('a' * 80, 'a' * 80, 'a' * 80),
    ('a' * 80, 'b' * 80, ''),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected


@pytest.mark.parametrize('seqs, expected', [
    (('a', 'b', 'c'), ''),
    (('a', 'a', 'a'), 'a'),
    (('test', 'text', 'tempest'), 'tet'),
])
def test_distance_multiseq(seqs, expected):
    actual = ALG(external=False)(*seqs)
    assert actual == expected

    actual = ALG(external=True)(*seqs)
    assert actual == expected



# built-in
from fractions import Fraction
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.arith_ncd


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'test', 1),
    ('test', 'nani', 2.1666666666666665),
])
def test_similarity(left, right, expected):
    actual = ALG(left, right)
    assert isclose(actual, expected)


def test_make_probs():
    alg = textdistance.ArithNCD(terminator='\x00')
    probs = alg._make_probs('lol', 'lal')
    assert probs['l'] == (Fraction(0, 1), Fraction(4, 7))
    assert probs['o'][1] == Fraction(1, 7)
    assert probs['a'][1] == Fraction(1, 7)


def test_arith_output():
    alg = textdistance.ArithNCD(terminator='\x00')
    fraction = alg._compress('BANANA')
    assert fraction.numerator == 1525



# external
import pytest

# project
import textdistance


ALG = textdistance.LCSStr


@pytest.mark.parametrize('left, right, expected', [
    # prefix
    ('ab', 'abcd', 'ab'),
    ('abcd', 'ab', 'ab'),

    # middle
    ('abcd', 'bc', 'bc'),
    ('bc', 'abcd', 'bc'),

    # suffix
    ('abcd', 'cd', 'cd'),
    ('abcd', 'cd', 'cd'),

    # no
    ('abcd', 'ef', ''),
    ('ef', 'abcd', ''),

    # long
    # https://github.com/life4/textdistance/issues/40
    ('MYTEST' * 100, 'TEST', 'TEST'),
    ('TEST', 'MYTEST' * 100, 'TEST'),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.bwtrle_ncd


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'test', 0.6),
    ('test', 'nani', 0.8),
])
def test_similarity(left, right, expected):
    actual = ALG(left, right)
    assert isclose(actual, expected)



# built-in
from math import isclose

# external
import hypothesis
import pytest

# project
import textdistance


ALGS = (
    textdistance.arith_ncd,
    textdistance.bwtrle_ncd,
    textdistance.bz2_ncd,

    # too slow, makes CI flaky
    # textdistance.lzma_ncd,

    textdistance.rle_ncd,
    textdistance.zlib_ncd,
    textdistance.sqrt_ncd,
    textdistance.entropy_ncd,
)


@pytest.mark.parametrize('alg', ALGS)
def test_monotonicity(alg):
    same = alg('test', 'test')
    similar = alg('test', 'text')
    diffirent = alg('test', 'nani')
    assert same <= similar <= diffirent


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_simmetry(left, right, alg):
    assert alg.similarity(left, right) == alg.similarity(right, left)
    assert alg.distance(left, right) == alg.distance(right, left)
    assert alg.normalized_similarity(left, right) == alg.normalized_similarity(right, left)
    assert alg.normalized_distance(left, right) == alg.normalized_distance(right, left)


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_is_normalized(left, right, alg):
    a = alg(left, right)
    d = alg.distance(left, right)
    nd = alg.normalized_distance(left, right)
    assert a == d == nd


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_normalized_by_one(left, right, alg):
    s = alg.normalized_similarity(left, right)
    d = alg.normalized_distance(left, right)
    assert isclose(s + d, 1)



# built-in
from math import isclose

# external
import hypothesis
import pytest

# project
import textdistance


ALG = textdistance.entropy_ncd


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'test', 1),
    ('aaa', 'bbb', 0),
    ('test', 'nani', 0.6),
])
def test_similarity(left, right, expected):
    actual = ALG.similarity(left, right)
    assert isclose(actual, expected)


@hypothesis.given(text=hypothesis.strategies.text(min_size=1))
def test_simmetry_compressor(text):
    rev = ''.join(reversed(text))
    assert isclose(ALG._compress(text), ALG._compress(rev))


@hypothesis.given(text=hypothesis.strategies.text(min_size=1))
def test_idempotency_compressor(text):
    # I've modified idempotency to some kind of distributivity for constant.
    # Now it indicates that compressor really compress.
    assert ALG._get_size(text * 2) < ALG._get_size(text) * 2


@hypothesis.given(
    left=hypothesis.strategies.text(min_size=1),
    right=hypothesis.strategies.characters(),
)
def test_monotonicity_compressor(left, right):
    if right in left:
        return
    assert ALG._get_size(left) <= ALG._get_size(left + right)


@hypothesis.given(
    left1=hypothesis.strategies.text(min_size=1),
    left2=hypothesis.strategies.text(min_size=1),
    right=hypothesis.strategies.characters(),
)
def test_distributivity_compressor(left1, left2, right):
    if right in left1 or right in left2:
        return
    actual1 = ALG._get_size(left1 + left2) + ALG._get_size(right)
    actual2 = ALG._get_size(left1 + right) + ALG._get_size(left2 + right)
    assert actual1 <= actual2


@hypothesis.given(text=hypothesis.strategies.text(min_size=1))
def test_normalization_range(text):
    assert 0 <= ALG.normalized_similarity(text, text) <= 1
    assert 0 <= ALG.normalized_distance(text, text) <= 1



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.StrCmp95


@pytest.mark.parametrize('left, right, expected', [
    ('MARTHA', 'MARHTA', 0.9611111111111111),
    ('DWAYNE', 'DUANE', 0.873),
    ('DIXON', 'DICKSONX', 0.839333333),
    ('TEST', 'TEXT', 0.9066666666666666),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert isclose(actual, expected)

    actual = ALG(external=True)(left, right)
    assert isclose(actual, expected)



# external
import pytest

# project
import textdistance


ALG = textdistance.Matrix
# https://en.wikipedia.org/wiki/Needleman%E2%80%93Wunsch_algorithm
NW_MATRIX = {
    ('A', 'A'): 10,
    ('G', 'G'): 7,
    ('C', 'C'): 9,
    ('T', 'T'): 8,
    ('A', 'G'): -1,
    ('A', 'C'): -3,
    ('A', 'T'): -4,
    ('G', 'C'): -5,
    ('G', 'T'): -3,
    ('C', 'T'): 0,
}


@pytest.mark.parametrize('left, right, expected', [
    ('', '', 1),
    ('', 'a', 0),
    ('abcd', 'abcd', 1),
    ('A', 'C', -3),
    ('G', 'G', 7),
    ('A', 'A', 10),
    ('T', 'A', -4),
    ('T', 'C', 0),
    ('A', 'G', -1),
    ('C', 'T', 0),
])
def test_distance(left, right, expected):
    actual = ALG(NW_MATRIX, symmetric=True)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import hypothesis
import pytest

# project
import textdistance


ALGS = (
    textdistance.bag,

    textdistance.hamming,
    textdistance.levenshtein,
    textdistance.damerau_levenshtein,
    textdistance.jaro,
    textdistance.jaro_winkler,
    textdistance.mlipns,

    textdistance.lcsseq,
    textdistance.lcsstr,
    textdistance.ratcliff_obershelp,

    textdistance.jaccard,
    textdistance.sorensen,
    textdistance.tversky,
    textdistance.overlap,
    textdistance.cosine,
    textdistance.strcmp95,
    textdistance.monge_elkan,

    textdistance.mra,

    textdistance.prefix,
    textdistance.postfix,
    textdistance.identity,
    # textdistance.length,

    # numpy-based:
    # textdistance.gotoh,
    textdistance.needleman_wunsch,
    textdistance.smith_waterman,
    textdistance.editex,
)


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_normalization_range(left, right, alg):
    assert 0 <= alg.normalized_distance(left, right) <= 1
    assert 0 <= alg.normalized_similarity(left, right) <= 1


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_normalization_by_one(left, right, alg):
    d = alg.normalized_distance(left, right)
    s = alg.normalized_similarity(left, right)
    assert isclose(s + d, 1)


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.given(text=hypothesis.strategies.text())
def test_normalization_same(text, alg):
    assert alg.normalized_distance(text, text) == 0
    if alg is not textdistance.needleman_wunsch:
        assert alg.distance(text, text) == 0
    assert alg.normalized_similarity(text, text) == 1


@pytest.mark.parametrize('alg', ALGS)
@hypothesis.settings(deadline=None)
@hypothesis.given(
    left=hypothesis.strategies.text(min_size=1),
    right=hypothesis.strategies.text(min_size=1),
)
def test_normalization_monotonic(left, right, alg):
    nd = alg.normalized_distance(left, right)
    ns = alg.normalized_similarity(left, right)
    d = alg.distance(left, right)
    s = alg.similarity(left, right)
    assert (nd < ns) == (d < s)


@pytest.mark.parametrize('alg', ALGS)
def test_no_common_chars(alg):
    if alg is textdistance.editex:
        return
    assert alg.similarity('spam', 'qwer') == 0


@pytest.mark.parametrize('alg', ALGS)
def test_empty(alg):
    assert alg.distance('', '') == 0


@pytest.mark.parametrize('alg', ALGS)
def test_unequal_distance(alg):
    if alg.maximum('', 'qwertyui'):
        assert alg.distance('', 'qwertyui') > 0



# external
import pytest

# project
import textdistance


ALG = textdistance.Editex


@pytest.mark.parametrize('left, right, expected', [
    ('', '', 0),
    ('nelson', '', 12),
    ('', 'neilsen', 14),
    ('ab', 'a', 2),
    ('ab', 'c', 4),
    ('ALIE', 'ALI', 1),
    ('', 'MARTHA', 12),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected


@pytest.mark.parametrize('left, right, params, expected', [
    ('MARTHA', 'MARHTA', dict(match_cost=2), 12),
    ('MARTHA', 'MARHTA', dict(match_cost=4), 24),
    ('MARTHA', 'MARHTA', dict(group_cost=1, local=True), 3),
    ('MARTHA', 'MARHTA', dict(group_cost=2, local=True), 4),
    ('MARTHA', 'MARHTA', dict(mismatch_cost=4, local=True), 5),
])
def test_distance_with_params(left, right, params, expected):
    actual = ALG(external=False, **params)(left, right)
    assert actual == expected

    actual = ALG(external=True, **params)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import hypothesis
import pytest

# project
import textdistance
from textdistance.libraries import prototype


libraries = prototype.clone()

# numpy throws a bunch of warning about abydos using `np.int` isntead of `int`.
ABYDOS_WARNINGS = (
    'ignore:`np.int` is a deprecated alias',
    'ignore:`np.float` is a deprecated alias',
    'ignore:Using or importing the ABCs',
)


@pytest.mark.filterwarnings(*ABYDOS_WARNINGS)
@pytest.mark.external
@pytest.mark.parametrize('alg', libraries.get_algorithms())
@hypothesis.settings(deadline=None)
@hypothesis.given(
    left=hypothesis.strategies.text(min_size=1),
    right=hypothesis.strategies.text(min_size=1),
)
def test_compare(left, right, alg):
    for lib in libraries.get_libs(alg):
        conditions = lib.conditions or {}
        internal_func = getattr(textdistance, alg)(external=False, **conditions)
        external_func = lib.get_function()
        if external_func is None:
            raise RuntimeError('cannot import {}'.format(str(lib)))

        if not lib.check_conditions(internal_func, left, right):
            continue

        int_result = internal_func(left, right)
        s1, s2 = lib.prepare(left, right)
        ext_result = external_func(s1, s2)
        assert isclose(int_result, ext_result), str(lib)


@pytest.mark.filterwarnings(*ABYDOS_WARNINGS)
@pytest.mark.external
@pytest.mark.parametrize('alg', libraries.get_algorithms())
@hypothesis.given(
    left=hypothesis.strategies.text(min_size=1),
    right=hypothesis.strategies.text(min_size=1),
)
@pytest.mark.parametrize('qval', (None, 1, 2, 3))
def test_qval(left, right, alg, qval):
    for lib in libraries.get_libs(alg):
        conditions = lib.conditions or {}
        internal_func = getattr(textdistance, alg)(external=False, **conditions)
        external_func = lib.get_function()
        if external_func is None:
            raise RuntimeError('cannot import {}'.format(str(lib)))
        # algorithm doesn't support q-grams
        if not hasattr(internal_func, 'qval'):
            continue

        internal_func.qval = qval
        # if qval unsopporting already set for lib
        s1, s2 = internal_func._get_sequences(left, right)
        if not lib.check_conditions(internal_func, s1, s2):
            continue
        quick_answer = internal_func.quick_answer(s1, s2)
        if quick_answer is not None:
            continue

        # test
        int_result = internal_func(left, right)
        s1, s2 = lib.prepare(s1, s2)
        ext_result = external_func(s1, s2)
        assert isclose(int_result, ext_result), f'{lib}({repr(s1)}, {repr(s2)})'


@pytest.mark.filterwarnings(*ABYDOS_WARNINGS)
@pytest.mark.external
@pytest.mark.parametrize('alg', libraries.get_algorithms())
@hypothesis.given(
    left=hypothesis.strategies.lists(hypothesis.strategies.integers()),
    right=hypothesis.strategies.lists(hypothesis.strategies.integers()),
)
def test_list_of_numbers(left, right, alg):
    for lib in libraries.get_libs(alg):
        conditions = lib.conditions or {}
        internal_func = getattr(textdistance, alg)(external=False, **conditions)
        external_func = lib.get_function()
        if external_func is None:
            raise RuntimeError('cannot import {}'.format(str(lib)))

        quick_answer = internal_func.quick_answer(left, right)
        if quick_answer is not None:
            continue
        if not lib.check_conditions(internal_func, left, right):
            continue

        int_result = internal_func(left, right)
        s1, s2 = lib.prepare(left, right)
        ext_result = external_func(s1, s2)
        assert isclose(int_result, ext_result), f'{lib}({repr(s1)}, {repr(s2)})'



# external
import pytest

# project
import textdistance


ALG = textdistance.MLIPNS


@pytest.mark.parametrize('left, right, expected', [
    ('', '', 1),
    ('a', '', 0),
    ('', 'a', 0),
    ('a', 'a', 1),
    ('ab', 'a', 1),
    ('abc', 'abc', 1),
    ('abc', 'abcde', 1),
    ('abcg', 'abcdeg', 1),
    ('abcg', 'abcdefg', 0),
    ('Tomato', 'Tamato', 1),
    ('ato', 'Tam', 1),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.MongeElkan


@pytest.mark.parametrize('left, right, expected', [
    (['Niall'], ['Neal'], .805),
    (['Niall'], ['Nigel'], 0.7866666666666667),
])
def test_similarity(left, right, expected):
    actual = ALG(qval=1, algorithm=textdistance.jaro_winkler).similarity(left, right)
    assert isclose(actual, expected)



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.Cosine


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'text', 3.0 / 4),
    ('nelson', 'neilsen', 5.0 / pow(6 * 7, .5)),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert isclose(actual, expected)

    actual = ALG(external=True)(left, right)
    assert isclose(actual, expected)



# built-in
from itertools import islice
from pathlib import Path
from sys import argv

# project
from textdistance import EntropyNCD


# read files
licenses = dict()
for path in Path('choosealicense.com', '_licenses').iterdir():
    licenses[path.stem] = path.read_text()

# show licenses list if no arguments passed
if len(argv) == 1:
    print(*sorted(licenses.keys()), sep='\n')
    exit(1)

# compare all with one
qval = int(argv[1]) if argv[1] else None
compare_with = argv[2]
distances = dict()
for name, content in licenses.items():
    distances[name] = EntropyNCD(qval=qval)(
        licenses[compare_with],
        content,
    )

# show 5 most similar
sorted_distances = sorted(distances.items(), key=lambda d: d[1])
for name, distance in islice(sorted_distances, 5):
    print('{:20} {:.4f}'.format(name, distance))



# external
import pytest

# project
import textdistance


ALG = textdistance.Editex


@pytest.mark.parametrize('left, right, expected', [
    # https://github.com/chrislit/abydos/blob/master/tests/distance/test_distance_editex.py
    ('', '', 0),
    ('nelson', '', len('nelson') * 2),
    ('', 'neilsen', len('neilsen') * 2),
    ('ab', 'a', 2),
    ('ab', 'c', 4),
    ('nelson', 'neilsen', 2),
    ('neilsen', 'nelson', 2),
    ('niall', 'neal', 1),
    ('neal', 'niall', 1),
    ('niall', 'nihal', 2),
    ('nihal', 'niall', 2),
    ('neal', 'nihl', 3),
    ('nihl', 'neal', 3),

    # https://anhaidgroup.github.io/py_stringmatching/v0.3.x/Editex.html
    ('cat', 'hat', 2),
    ('Niall', 'Neil', 2),
    ('aluminum', 'Catalan', 12),
    ('ATCG', 'TAGC', 6),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected


@pytest.mark.parametrize('left, right, expected', [
    ('', '', 0),
    ('nelson', '', 12),
    ('', 'neilsen', 14),
    ('ab', 'a', 2),
    ('ab', 'c', 2),
    ('nelson', 'neilsen', 2),
    ('neilsen', 'nelson', 2),
    ('niall', 'neal', 1),
    ('neal', 'niall', 1),
    ('niall', 'nihal', 2),
    ('nihal', 'niall', 2),
    ('neal', 'nihl', 3),
    ('nihl', 'neal', 3),
])
def test_local(left, right, expected):
    actual = ALG(external=False, local=True)(left, right)
    assert actual == expected

    actual = ALG(external=True, local=True)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.JaroWinkler


@pytest.mark.parametrize('left, right, expected', [
    ('elephant', 'hippo', 0.44166666666666665),
    ('fly', 'ant', 0.0),
    ('frog', 'fog', 0.925),
    ('MARTHA', 'MARHTA', 0.9611111111111111),
    ('DWAYNE', 'DUANE', 0.84),
    ('DIXON', 'DICKSONX', 0.8133333333333332),

    # https://github.com/life4/textdistance/issues/39
    ('duck donald', 'duck daisy', 0.867272727272),
])
def test_distance(left, right, expected):
    actual = ALG(winklerize=True, external=False)(left, right)
    assert isclose(actual, expected)

    actual = ALG(winklerize=True, external=True)(left, right)
    assert isclose(actual, expected)



# external
import pytest

# project
import textdistance


ALG = textdistance.Bag


@pytest.mark.parametrize('left, right, expected', [
    ('qwe', 'qwe', 0),
    ('qwe', 'erty', 3),
    ('qwe', 'ewq', 0),
    ('qwe', 'rtys', 4),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected



# external
import pytest

# project
import textdistance


ALG = textdistance.Gotoh
# https://en.wikipedia.org/wiki/Needleman%E2%80%93Wunsch_algorithm
NW_MATRIX = {
    ('A', 'A'): 10,
    ('G', 'G'): 7,
    ('C', 'C'): 9,
    ('T', 'T'): 8,
    ('A', 'G'): -1,
    ('A', 'C'): -3,
    ('A', 'T'): -4,
    ('G', 'C'): -5,
    ('G', 'T'): -3,
    ('C', 'T'): 0,
}


def sim_ident(x, y):
    if x == y:
        return 1
    else:
        return -1


@pytest.mark.parametrize('left, right, expected', [
    ('GATTACA', 'GCATGCU', 0),
])
def test_distance_ident(left, right, expected):
    actual = ALG(gap_open=1, gap_ext=1, sim_func=sim_ident)(left, right)
    assert actual == expected


@pytest.mark.parametrize('left, right, expected', [
    ('GATTACA', 'GCATGCU', 0),
    ('AGACTAGTTAC', 'TGACGSTGC', 1.5),
    ('AGACTAGTTAC', 'CGAGACGT', 1),
])
def test_distance_ident_with_gap_05(left, right, expected):
    actual = ALG(gap_open=1, gap_ext=.5, sim_func=sim_ident)(left, right)
    assert actual == expected


@pytest.mark.parametrize('left, right, expected', [
    ('AGACTAGTTAC', 'CGAGACGT', -15),
])
def test_distance_ident_with_gap_5(left, right, expected):
    actual = ALG(gap_open=5, gap_ext=5, sim_func=sim_ident)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import pytest

# project
import textdistance


ALG = textdistance.JaroWinkler


@pytest.mark.parametrize('left, right, expected', [
    ('hello', 'haloa', 0.7333333333333334),
    ('fly', 'ant', 0.0),
    ('frog', 'fog', 0.9166666666666666),
    ('ATCG', 'TAGC', 0.8333333333333334),
    ('MARTHA', 'MARHTA', 0.944444444),
    ('DWAYNE', 'DUANE', 0.822222222),
    ('DIXON', 'DICKSONX', 0.7666666666666666),

    # https://github.com/life4/textdistance/issues/41
    ('Sint-Pietersplein 6, 9000 Gent', 'Test 10, 1010 Brussel', 0.5182539682539683),
])
def test_distance(left, right, expected):
    actual = ALG(winklerize=False, external=False)(left, right)
    assert isclose(actual, expected)

    actual = ALG(winklerize=False, external=True)(left, right)
    assert isclose(actual, expected)



# external
import pytest

# project
import textdistance


ALG = textdistance.Hamming


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'text', 1),
    ('test', 'tset', 2),
    ('test', 'qwe', 4),
    ('test', 'testit', 2),
    ('test', 'tesst', 2),
    ('test', 'tet', 2),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert actual == expected

    actual = ALG(external=True)(left, right)
    assert actual == expected



# built-in
from math import isclose

# external
import hypothesis
import pytest

# project
import textdistance


ALG = textdistance.Jaccard


@pytest.mark.parametrize('left, right, expected', [
    ('test', 'text', 3.0 / 5),
    ('nelson', 'neilsen', 5.0 / 8),
    ('decide', 'resize', 3.0 / 9),
])
def test_distance(left, right, expected):
    actual = ALG(external=False)(left, right)
    assert isclose(actual, expected)

    actual = ALG(external=True)(left, right)
    assert isclose(actual, expected)


@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_compare_with_tversky(left, right):
    td = textdistance.Tversky(ks=[1, 1]).distance(left, right)
    jd = ALG().distance(left, right)
    assert isclose(jd, td)


@hypothesis.given(
    left=hypothesis.strategies.text(),
    right=hypothesis.strategies.text(),
)
def test_compare_with_tversky_as_set(left, right):
    td = textdistance.Tversky(ks=[1, 1], as_set=True).distance(left, right)
    jd = ALG(as_set=True).distance(left, right)
    assert isclose(jd, td)
